@@ -499,8 +499,8 @@ class Interpreter:
         else:
             fnScope.RunFunction = self.RunFunction
             self.InterpretExternalFunction(func, fnScope)
-        if returnVar.Type == types.VariableTypes.Nil and returnVar.Value.Value != types.VariableTypes.Nil:
-            raise exceptions.RuntimeError(f"ExtFunction ({name}) returned nil type without wrong value ({returnVar.Value}).", tok.Location, self.filename)
+        if returnVar.Value.Type == types.VariableTypes.Nil and returnVar.Value.Value != types.VariableTypes.Nil:
+            raise exceptions.RuntimeError(f"ExtFunction ({name}) returned nil type with a wrong value ({returnVar.Value}).", tok.Location, self.filename)
         return returnVar.Value
 
 
@@ -688,10 +688,14 @@ class Interpreter:
             raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr is not a value item and it should be.', None, self.filename)
         if isinstance(arr.Value, value_item.ValueItem):
             raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr.Value is a value item and it should not be.', None, self.filename)
+        if arr.Type == types.VariableTypes.Nil and arr.Value != types.VariableTypes.Nil:
+            raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr.Type is nil but arr.Value is not.', None, self.filename)
         if arr.Type == types.VariableTypes.Array and len(arr.Value) > 0:
             for rec in arr.Value:
                 if not isinstance(rec, value_item.ValueItem):
                     raise exceptions.RuntimeError(f'checkDataArray ({msg}), arr.Value[n] is not a value item and it should be.', None, self.filename)
+                if rec.Type == types.VariableTypes.Nil and rec.Value != types.VariableTypes.Nil:
+                    raise exceptions.RuntimeError(f'checkDataArray ({msg}), rec.Type is nil but rec.Value is not.', None, self.filename)
 
 
 
@@ -699,6 +703,7 @@ class Interpreter:
         data = self.InterpretExpression(instruction.From, scope)
         self.checkDataArray(data, 'data after init')
         into = self.EvaluateIntoVariable(instruction.Into, data, scope)
+        self.checkDataArray(into, f'into after EvaluateIntoVariable, {instruction.Into}')
         if into is None:
             if instruction.Merge is True:
                 raise exceptions.RuntimeError(f'Variable ({instruction.Into.Token.Lexeme}) not found.', instruction.Into.Token.Location, self.filename)
@@ -709,15 +714,13 @@ class Interpreter:
                 intoVar.Value = value_item.ValueItem().SetFromMacal(instruction.Token, types.VariableTypes.Array, [])
             scope.AddVariable(intoVar)
             into = intoVar.Value      
-        self.checkDataArray(into, 'into after init')
+            self.checkDataArray(into, 'into from new intoVar after into was none')
         if instruction.Where is not None and data is not None:
             data = self.InterpretSelectWhere(instruction.Where, data, scope)
         self.checkDataArray(data, 'data after where')
-
         if not(len(instruction.Fields) == 1 and instruction.Fields[0].Name == '*'):
             data = self.ApplyFieldFilters(data, instruction.Fields)
         self.checkDataArray(data, 'data after apply field filters')
-
         if instruction.Distinct is True and data is not None and data.Type == types.VariableTypes.Array and len(data.Value) > 0:
             if isinstance(data.Value[0], value_item.ValueItem):
                 data = data.Value[0]
@@ -1007,8 +1010,7 @@ class Interpreter:
             i = self.InterpretExpression(ind, scope)
             index_as_str = f'{index_as_str}[{i.Value}]'
             if val.Type not in [types.VariableTypes.Array, types.VariableTypes.Record, types.VariableTypes.String]:
-                raise exceptions.RuntimeError(f'Index type ({val.Type}) error. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
-            
+                raise exceptions.RuntimeError(f'Index type ({val.Type}) error. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)            
             if val.Type == types.VariableTypes.Record:
                 if i.Value not in val.Value:
                     raise exceptions.RuntimeError(f'Index not found. ({var.Name}{index_as_str}).', ind.Token.Location, self.filename)
